@@ -11,7 +11,6 @@ use Icinga\Module\Director\Db\Cache\PrefetchCache;
 use Icinga\Module\Director\Db;
 use Icinga\Module\Director\Hook\ShipConfigFilesHook;
 use Icinga\Module\Director\Objects\IcingaObject;
-use Icinga\Module\Director\Util;
 use Icinga\Module\Director\Objects\IcingaHost;
 use Icinga\Module\Director\Objects\IcingaZone;
 use InvalidArgumentException;
@@ -270,7 +269,7 @@ class IcingaConfig
         $checksum = $this->calculateChecksum();
         $activity = $this->getLastActivityChecksum();
 
-        $lastActivity = $this->binFromDb(
+        $lastActivity = $this->connection->binaryDbResult(
             $this->db->fetchOne(
                 $this->db->select()->from(
                     self::$table,
@@ -310,20 +309,7 @@ class IcingaConfig
 
     protected function dbBin($binary)
     {
-        if ($this->connection->isPgsql()) {
-            return Util::pgBinEscape($binary);
-        } else {
-            return $binary;
-        }
-    }
-
-    protected function binFromDb($value)
-    {
-        if (is_resource($value)) {
-            return stream_get_contents($value);
-        }
-
-        return $value;
+        return $this->connection->quoteBinary($binary);
     }
 
     protected function calculateChecksum()
@@ -599,9 +585,9 @@ if (! globals.contains(DirectorOverrideTemplate)) {
             throw new NotFoundError('Got no config for %s', bin2hex($checksum));
         }
 
-        $this->checksum = $this->binFromDb($result->checksum);
+        $this->checksum = $this->connection->binaryDbResult($result->checksum);
         $this->generationTime = $result->duration;
-        $this->lastActivityChecksum = $this->binFromDb($result->last_activity_checksum);
+        $this->lastActivityChecksum = $this->connection->binaryDbResult($result->last_activity_checksum);
 
         $query = $this->db->select()->from(
             array('cf' => 'director_generated_config_file'),
