@@ -7,7 +7,10 @@ use Icinga\Application\Benchmark;
 use Icinga\Data\Filter\Filter;
 use Icinga\Module\Director\Application\MemoryLimit;
 use Icinga\Module\Director\Data\Db\DbObject;
+use Icinga\Module\Director\Data\Db\DbObjectStore;
+use Icinga\Module\Director\Data\Db\DbObjectTypeRegistry;
 use Icinga\Module\Director\Db;
+use Icinga\Module\Director\Db\Branch\BranchSupport;
 use Icinga\Module\Director\Db\Cache\PrefetchCache;
 use Icinga\Module\Director\Objects\HostGroupMembershipResolver;
 use Icinga\Module\Director\Objects\IcingaHost;
@@ -21,6 +24,7 @@ use Icinga\Module\Director\Objects\SyncRun;
 use Icinga\Exception\IcingaException;
 use Icinga\Module\Director\Repository\IcingaTemplateRepository;
 use InvalidArgumentException;
+use RuntimeException;
 
 class Sync
 {
@@ -42,9 +46,15 @@ class Sync
     /** @var IcingaObject[] Objects to work with */
     protected $objects;
 
+    /** @var array<mixed, array<int, string>> key => [property, property]*/
+    protected $setNull = [];
+
     /** @var bool Whether we already prepared your sync */
     protected $isPrepared = false;
 
+    /** @var bool Whether we applied strtolower() to existing object keys */
+    protected $usedLowerCasedKeys = false;
+
     protected $modify = [];
 
     protected $remove = [];
@@ -75,13 +85,18 @@ class Sync
     /** @var HostGroupMembershipResolver|bool */
     protected $hostGroupMembershipResolver;
 
+    /** @var ?DbObjectStore */
+    protected $store;
+
     /**
      * @param SyncRule $rule
+     * @param ?DbObjectStore $store
      */
-    public function __construct(SyncRule $rule)
+    public function __construct(SyncRule $rule, DbObjectStore $store = null)
     {
         $this->rule = $rule;
         $this->db = $rule->getConnection();
+        $this->store = $store;
     }
 
     /**
@@ -183,7 +198,7 @@ class Sync
                 $this->hasPropertyDisabled = true;
             }
 
-            if (! strlen($prop->get('filter_expression'))) {
+            if ($prop->get('filter_expression') === null || strlen($prop->get('filter_expression')) === 0) {
                 continue;
             }
 
@@ -305,6 +320,9 @@ class Sync
             foreach ($rows as $row) {
                 if ($combinedKey) {
                     $key = SyncUtils::fillVariables($sourceKeyPattern, $row);
+                    if ($this->usedLowerCasedKeys) {
+                        $key = strtolower($key);
+                    }
 
                     if (array_key_exists($key, $this->imported[$sourceId])) {
                         throw new InvalidArgumentException(sprintf(
@@ -333,7 +351,11 @@ class Sync
                 if ($combinedKey) {
                     $this->imported[$sourceId][$key] = $row;
                 } else {
-                    $this->imported[$sourceId][$row->$key] = $row;
+                    if ($this->usedLowerCasedKeys) {
+                        $this->imported[$sourceId][strtolower($row->$key)] = $row;
+                    } else {
+                        $this->imported[$sourceId][$row->$key] = $row;
+                    }
                 }
             }
 
@@ -359,13 +381,13 @@ class Sync
 
         if ($listId === null) {
             throw new InvalidArgumentException(
-                'Cannot sync datalist entry without list_ist'
+                'Cannot sync datalist entry without list_id'
             );
         }
 
         $no = [];
         foreach ($this->objects as $k => $o) {
-            if ((int) $o->get('list_id') !== (int) $listId) {
+            if ((int) $o->get('list_id') !== $listId) {
                 $no[] = $k;
             }
         }
@@ -383,15 +405,19 @@ class Sync
         Benchmark::measure('Begin loading existing objects');
 
         $ruleObjectType = $this->rule->get('object_type');
+        $useLowerCaseKeys = $ruleObjectType !== 'datalistEntry';
         // TODO: Make object_type (template, object...) and object_name mandatory?
         if ($this->rule->hasCombinedKey()) {
             $this->objects = [];
             $destinationKeyPattern = $this->rule->getDestinationKeyPattern();
+            $table = DbObjectTypeRegistry::tableNameByType($ruleObjectType);
+            if ($this->store && BranchSupport::existsForTableName($table)) {
+                $objects = $this->store->loadAll($table);
+            } else {
+                $objects = IcingaObject::loadAllByType($ruleObjectType, $this->db);
+            }
 
-            foreach (IcingaObject::loadAllByType(
-                $ruleObjectType,
-                $this->db
-            ) as $object) {
+            foreach ($objects as $object) {
                 if ($object instanceof IcingaService) {
                     if (strstr($destinationKeyPattern, '${host}')
                         && $object->get('host_id') === null
@@ -408,6 +434,9 @@ class Sync
                     $destinationKeyPattern,
                     $object
                 );
+                if ($useLowerCaseKeys) {
+                    $key = strtolower($key);
+                }
 
                 if (array_key_exists($key, $this->objects)) {
                     throw new InvalidArgumentException(sprintf(
@@ -420,12 +449,23 @@ class Sync
                 $this->objects[$key] = $object;
             }
         } else {
-            $this->objects = IcingaObject::loadAllByType(
-                $ruleObjectType,
-                $this->db
-            );
+            if ($this->store) {
+                $objects = $this->store->loadAll(DbObjectTypeRegistry::tableNameByType($ruleObjectType), 'object_name');
+            } else {
+                $objects = IcingaObject::loadAllByType($ruleObjectType, $this->db);
+            }
+
+            if ($useLowerCaseKeys) {
+                $this->objects = [];
+                foreach ($objects as $key => $object) {
+                    $this->objects[strtolower($key)] = $object;
+                }
+            } else {
+                $this->objects = $objects;
+            }
         }
 
+        $this->usedLowerCasedKeys = $useLowerCaseKeys;
         // TODO: should be obsoleted by a better "loadFiltered" method
         if ($ruleObjectType === 'datalistEntry') {
             $this->removeForeignListEntries();
@@ -448,10 +488,15 @@ class Sync
 
         foreach ($this->sources as $source) {
             $sourceId = $source->id;
+            $keyColumn = $source->get('key_column');
 
             foreach ($this->imported[$sourceId] as $key => $row) {
                 // Workaround: $a["10"] = "val"; -> array_keys($a) = [(int) 10]
                 $key = (string) $key;
+                $originalKey = $row->$keyColumn;
+                if ($this->usedLowerCasedKeys) {
+                    $key = strtolower($key);
+                }
                 if (! array_key_exists($key, $objects)) {
                     // Safe default values for object_type and object_name
                     if ($ruleObjectType === 'datalistEntry') {
@@ -459,7 +504,7 @@ class Sync
                     } else {
                         $props = [
                             'object_type' => 'object',
-                            'object_name' => $key
+                            'object_name' => $originalKey,
                         ];
                     }
 
@@ -471,7 +516,7 @@ class Sync
                 }
 
                 $object = $objects[$key];
-                $this->prepareNewObject($row, $object, $sourceId);
+                $this->prepareNewObject($row, $object, $key, $sourceId);
             }
         }
 
@@ -485,7 +530,7 @@ class Sync
      * @throws \Icinga\Exception\NotFoundError
      * @throws \Icinga\Module\Director\Exception\DuplicateKeyException
      */
-    protected function prepareNewObject($row, DbObject $object, $sourceId)
+    protected function prepareNewObject($row, DbObject $object, $objectKey, $sourceId)
     {
         foreach ($this->syncProperties as $propertyKey => $p) {
             if ($p->get('source_id') !== $sourceId) {
@@ -497,7 +542,6 @@ class Sync
             }
 
             $prop = $p->get('destination_field');
-
             $val = SyncUtils::fillVariables($p->get('source_expression'), $row);
 
             if ($object instanceof IcingaObject) {
@@ -519,15 +563,26 @@ class Sync
                             $this->wantArray($val)
                         );
                     } else {
-                        $object->vars()->$varName = $val;
+                        if ($val === null) {
+                            $this->setNull[$objectKey][$prop] = $prop;
+                        } else {
+                            unset($this->setNull[$objectKey][$prop]);
+                            $object->vars()->$varName = $val;
+                        }
                     }
                 } else {
-                    if ($val !== null) {
+                    if ($val === null) {
+                        $this->setNull[$objectKey][$prop] = $prop;
+                    } else {
+                        unset($this->setNull[$objectKey][$prop]);
                         $object->set($prop, $val);
                     }
                 }
             } else {
-                if ($val !== null) {
+                if ($val === null) {
+                    $this->setNull[$objectKey][$prop] = $prop;
+                } else {
+                    unset($this->setNull[$objectKey][$prop]);
                     $object->set($prop, $val);
                 }
             }
@@ -633,7 +688,9 @@ class Sync
 
         Benchmark::measure('Modified objects are ready, applying purge strategy');
         $noAction = [];
+        $purgeAction = $this->rule->get('purge_action');
         foreach ($this->rule->purgeStrategy()->listObjectsToPurge() as $key) {
+            $key = strtolower($key);
             if (array_key_exists($key, $newObjects)) {
                 // Object has been touched, do not delete
                 continue;
@@ -642,7 +699,18 @@ class Sync
             if (array_key_exists($key, $this->objects)) {
                 $object = $this->objects[$key];
                 if (! $object->hasBeenModified()) {
-                    $object->markForRemoval();
+                    switch ($purgeAction) {
+                        case 'delete':
+                            $object->markForRemoval();
+                            break;
+                        case 'disable':
+                            $object->set('disabled', 'y');
+                            break;
+                        default:
+                            throw new RuntimeException(
+                                "Unsupported purge action: '$purgeAction'"
+                            );
+                    }
                 }
             }
         }
@@ -691,7 +759,13 @@ class Sync
 
         switch ($policy) {
             case 'override':
-                $this->objects[$key]->replaceWith($object);
+                if ($object instanceof IcingaHost
+                    && !in_array('api_key', $this->rule->getSyncProperties())
+                ) {
+                    $this->objects[$key]->replaceWith($object, ['api_key']);
+                } else {
+                    $this->objects[$key]->replaceWith($object);
+                }
                 break;
 
             case 'merge':
@@ -716,6 +790,12 @@ class Sync
                 }
             }
         }
+
+        if (isset($this->setNull[$key])) {
+            foreach ($this->setNull[$key] as $property) {
+                $this->objects[$key]->set($property, null);
+            }
+        }
     }
 
     /**
@@ -740,7 +820,9 @@ class Sync
         $objects = $this->prepare();
         $db = $this->db;
         $dba = $db->getDbAdapter();
-        $dba->beginTransaction();
+        if (! $this->store) { // store has it's own transaction
+            $dba->beginTransaction();
+        }
 
         $object = null;
         $updateOnly = $this->rule->get('update_policy') === 'update-only';
@@ -758,7 +840,11 @@ class Sync
             foreach ($objects as $object) {
                 $this->setResolver($object);
                 if (! $updateOnly && $object->shouldBeRemoved()) {
-                    $object->delete();
+                    if ($this->store) {
+                        $this->store->delete($object);
+                    } else {
+                        $object->delete();
+                    }
                     $deleted++;
                     continue;
                 }
@@ -766,10 +852,18 @@ class Sync
                 if ($object->hasBeenModified()) {
                     $existing = $object->hasBeenLoadedFromDb();
                     if ($existing) {
-                        $object->store($db);
+                        if ($this->store) {
+                            $this->store->store($object);
+                        } else {
+                            $object->store($db);
+                        }
                         $modified++;
                     } elseif ($allowCreate) {
-                        $object->store($db);
+                        if ($this->store) {
+                            $this->store->store($object);
+                        } else {
+                            $object->store($db);
+                        }
                         $created++;
                     }
                 }
@@ -789,24 +883,34 @@ class Sync
                 ));
             }
 
-            $this->run->setProperties($runProperties)->store();
+            $this->run->setProperties($runProperties);
+            if (!$this->store || !$this->store->getBranch()->isBranch()) {
+                $this->run->store();
+            }
             $this->notifyResolvers();
-            $dba->commit();
+            if (! $this->store) {
+                $dba->commit();
+            }
 
             // Store duration after commit, as the commit might take some time
             $this->run->set('duration_ms', (int) round(
                 (microtime(true) - $this->runStartTime) * 1000
-            ))->store();
+            ));
+            if (!$this->store || !$this->store->getBranch()->isBranch()) {
+                $this->run->store();
+            }
 
             Benchmark::measure('Done applying objects');
         } catch (Exception $e) {
-            $dba->rollBack();
+            if (! $this->store) {
+                $dba->rollBack();
+            }
 
-            if ($object !== null && $object instanceof IcingaObject) {
+            if ($object instanceof IcingaObject) {
                 throw new IcingaException(
                     'Exception while syncing %s %s: %s',
                     get_class($object),
-                    $object->get('object_name'),
+                    $object->getObjectName(),
                     $e->getMessage(),
                     $e
                 );
@@ -820,6 +924,9 @@ class Sync
 
     protected function prepareCache()
     {
+        if ($this->store) {
+            return $this;
+        }
         PrefetchCache::initialize($this->db);
         IcingaTemplateRepository::clear();
 
