@@ -14,6 +14,7 @@ class IcingaNotification extends IcingaObject implements ExportInterface
 
     protected $defaultProperties = [
         'id'                    => null,
+        'uuid'                  => null,
         'object_name'           => null,
         'object_type'           => null,
         'disabled'              => 'n',
@@ -31,6 +32,8 @@ class IcingaNotification extends IcingaObject implements ExportInterface
         'assign_filter'         => null,
     ];
 
+    protected $uuidColumn = 'uuid';
+
     protected $supportsCustomVars = true;
 
     protected $supportsFields = true;
@@ -95,6 +98,7 @@ class IcingaNotification extends IcingaObject implements ExportInterface
 
     /**
      * @return \stdClass
+     * @deprecated please use \Icinga\Module\Director\Data\Exporter
      * @throws \Icinga\Exception\NotFoundError
      */
     public function export()
@@ -139,6 +143,10 @@ class IcingaNotification extends IcingaObject implements ExportInterface
         return $object;
     }
 
+    /**
+     * @deprecated please use \Icinga\Module\Director\Data\FieldReferenceLoader
+     * @return array
+     */
     protected function loadFieldReferences()
     {
         $db = $this->getDb();
@@ -184,7 +192,7 @@ class IcingaNotification extends IcingaObject implements ExportInterface
         if ($this->isApplyRule()) {
             if (($to = $this->get('apply_to')) === null) {
                 throw new RuntimeException(sprintf(
-                    'Applied notification "%s" has no valid object type',
+                    'No "apply_to" object type has been set for Applied notification "%s"',
                     $this->getObjectName()
                 ));
             }
