@@ -4,6 +4,7 @@ namespace Icinga\Module\Director\Web\Table;
 
 use Icinga\Authentication\Auth;
 use Icinga\Module\Director\Db;
+use Icinga\Module\Director\Db\DbSelectParenthesis;
 use Icinga\Module\Director\Db\IcingaObjectFilterHelper;
 use Icinga\Module\Director\Objects\IcingaObject;
 use Icinga\Module\Director\Restriction\FilterByNameRestriction;
@@ -12,17 +13,21 @@ use Icinga\Module\Director\Restriction\ObjectRestriction;
 use gipfl\IcingaWeb2\Link;
 use gipfl\IcingaWeb2\Table\ZfQueryBasedTable;
 use gipfl\IcingaWeb2\Url;
+use Ramsey\Uuid\Uuid;
 use Zend_Db_Select as ZfSelect;
 
 class ObjectsTable extends ZfQueryBasedTable
 {
+    use TableWithBranchSupport;
+
     /** @var ObjectRestriction[] */
     protected $objectRestrictions;
 
     protected $columns = [
         'object_name' => 'o.object_name',
+        'object_type' => 'o.object_type',
         'disabled'    => 'o.disabled',
-        'id'          => 'o.id',
+        'uuid'        => 'o.uuid',
     ];
 
     protected $searchColumns = ['o.object_name'];
@@ -38,6 +43,10 @@ class ObjectsTable extends ZfQueryBasedTable
     /** @var IcingaObject */
     protected $dummyObject;
 
+    protected $leftSubQuery;
+
+    protected $rightSubQuery;
+
     /** @var Auth */
     private $auth;
 
@@ -134,7 +143,7 @@ class ObjectsTable extends ZfQueryBasedTable
     {
         $type = $this->baseObjectUrl;
         $url = Url::fromPath("director/${type}", [
-            'name' => $row->object_name
+            'uuid' => Uuid::fromBytes($row->uuid)->toString()
         ]);
 
         return static::td(Link::create($this->getMainLinkLabel($row), $url));
@@ -154,6 +163,9 @@ class ObjectsTable extends ZfQueryBasedTable
 
     public function renderRow($row)
     {
+        if (isset($row->uuid) && is_resource($row->uuid)) {
+            $row->uuid = stream_get_contents($row->uuid);
+        }
         $tr = static::tr([
             $this->renderObjectNameColumn($row),
             $this->renderExtraColumns($row)
@@ -172,11 +184,21 @@ class ObjectsTable extends ZfQueryBasedTable
 
     protected function getRowClasses($row)
     {
+        // TODO: remove isset, to figure out where it is missing
+        if (isset($row->branch_uuid) && $row->branch_uuid !== null) {
+            return ['branch_modified'];
+        }
         return [];
     }
 
-    protected function applyObjectTypeFilter(ZfSelect $query)
+    protected function applyObjectTypeFilter(ZfSelect $query, ZfSelect $right = null)
     {
+        if ($right) {
+            $right->where(
+                'bo.object_type = ?',
+                $this->filterObjectType
+            );
+        }
         return $query->where(
             'o.object_type = ?',
             $this->filterObjectType
@@ -203,6 +225,7 @@ class ObjectsTable extends ZfQueryBasedTable
 
     protected function loadRestrictions()
     {
+        /** @var Db $db */
         $db = $this->connection();
         $auth = $this->getAuth();
 
@@ -227,16 +250,66 @@ class ObjectsTable extends ZfQueryBasedTable
     protected function prepareQuery()
     {
         $table = $this->getDummyObject()->getTableName();
-        $query = $this->applyRestrictions(
-            $this->db()
-                ->select()
-                ->from(
-                    ['o' => $table],
-                    $this->getColumns()
-                )
-                ->order('o.object_name')->limit(100)
-        );
+        if ($this->branchUuid) {
+            $this->columns['branch_uuid'] = 'bo.branch_uuid';
+        }
 
-        return $this->applyObjectTypeFilter($query);
+        $columns = $this->getColumns();
+        if ($this->branchUuid) {
+            $columns = $this->branchifyColumns($columns);
+            $this->stripSearchColumnAliases();
+        }
+        $query = $this->db()->select()->from(['o' => $table], $columns);
+
+        if ($this->branchUuid) {
+            $right = clone($query);
+            // Hint: Right part has only those with object = null
+            //       This means that restrictions on $right would hide all
+            //       new rows. Dedicated restriction logic for the branch-only
+            //       part of thw union are not required, we assume that restrictions
+            //       for new objects have been checked once they have been created
+            $query = $this->applyRestrictions($query);
+            /** @var Db $conn */
+            $conn = $this->connection();
+            $query->joinLeft(
+                ['bo' => "branched_$table"],
+                // TODO: PgHexFunc
+                $this->db()->quoteInto(
+                    'bo.uuid = o.uuid AND bo.branch_uuid = ?',
+                    $conn->quoteBinary($this->branchUuid->getBytes())
+                ),
+                []
+            )->where("(bo.branch_deleted IS NULL OR bo.branch_deleted = 'n')");
+            $this->applyObjectTypeFilter($query, $right);
+            $right->joinRight(
+                ['bo' => "branched_$table"],
+                'bo.uuid = o.uuid',
+                []
+            )
+            ->where('o.uuid IS NULL')
+            ->where('bo.branch_uuid = ?', $conn->quoteBinary($this->branchUuid->getBytes()));
+            $this->leftSubQuery = $query;
+            $this->rightSubQuery = $right;
+            $query = $this->db()->select()->union([
+                'l' => new DbSelectParenthesis($query),
+                'r' => new DbSelectParenthesis($right),
+            ]);
+            $query = $this->db()->select()->from(['u' => $query]);
+            $query->order('object_name')->limit(100);
+        } else {
+            $this->applyObjectTypeFilter($query);
+            $query->order('o.object_name')->limit(100);
+        }
+
+        return $query;
+    }
+
+    public function removeQueryLimit()
+    {
+        $query = $this->getQuery();
+        $query->reset($query::LIMIT_OFFSET);
+        $query->reset($query::LIMIT_COUNT);
+
+        return $this;
     }
 }
